@@ -1,6 +1,6 @@
 import base64
 import click
-from click_default_group import DefaultGroup
+from click_default_group import DefaultGroup  # type: ignore
 from datetime import datetime
 import hashlib
 import pathlib
@@ -14,7 +14,17 @@ import os
 import sys
 import csv as csv_std
 import tabulate
-from .utils import file_progress, find_spatialite, sqlite3, decode_base64_values
+from .utils import (
+    file_progress,
+    find_spatialite,
+    sqlite3,
+    decode_base64_values,
+    rows_from_file,
+    Format,
+    TypeTracker,
+)
+
+CONTEXT_SETTINGS = dict(help_option_names=["-h", "--help"])
 
 VALID_COLUMN_TYPES = ("INTEGER", "TEXT", "FLOAT", "BLOB")
 
@@ -89,7 +99,12 @@ def load_extension_option(fn):
     )(fn)
 
 
-@click.group(cls=DefaultGroup, default="query", default_if_no_args=True)
+@click.group(
+    cls=DefaultGroup,
+    default="query",
+    default_if_no_args=True,
+    context_settings=CONTEXT_SETTINGS,
+)
 @click.version_option()
 def cli():
     "Commands for interacting with a SQLite database"
@@ -245,17 +260,6 @@ def views(
     )
 
 
-@cli.command()
-@click.argument(
-    "path",
-    type=click.Path(exists=True, file_okay=True, dir_okay=False, allow_dash=False),
-    required=True,
-)
-def vacuum(path):
-    """Run VACUUM against the database"""
-    sqlite_utils.Database(path).vacuum()
-
-
 @cli.command()
 @click.argument(
     "path",
@@ -308,6 +312,21 @@ def vacuum(path):
     sqlite_utils.Database(path).vacuum()
 
 
+@cli.command()
+@click.argument(
+    "path",
+    type=click.Path(exists=True, file_okay=True, dir_okay=False, allow_dash=False),
+    required=True,
+)
+@load_extension_option
+def dump(path, load_extension):
+    """Output a SQL dump of the schema and full contents of the database"""
+    db = sqlite_utils.Database(path)
+    _load_extensions(db, load_extension)
+    for line in db.conn.iterdump():
+        click.echo(line)
+
+
 @cli.command(name="add-column")
 @click.argument(
     "path",
@@ -654,6 +673,13 @@ def insert_upsert_options(fn):
                 "--encoding",
                 help="Character encoding for input, defaults to utf-8",
             ),
+            click.option(
+                "-d",
+                "--detect-types",
+                is_flag=True,
+                envvar="SQLITE_UTILS_DETECT_TYPES",
+                help="Detect types for columns in CSV/TSV data",
+            ),
             load_extension_option,
             click.option("--silent", is_flag=True, help="Do not show progress bar"),
         )
@@ -683,6 +709,7 @@ def insert_upsert_implementation(
     not_null=None,
     default=None,
     encoding=None,
+    detect_types=None,
     load_extension=None,
     silent=False,
 ):
@@ -694,11 +721,12 @@ def insert_upsert_implementation(
         raise click.ClickException("Use just one of --nl, --csv or --tsv")
     if encoding and not (csv or tsv):
         raise click.ClickException("--encoding must be used with --csv or --tsv")
+    if pk and len(pk) == 1:
+        pk = pk[0]
     encoding = encoding or "utf-8-sig"
     buffered = io.BufferedReader(json_file, buffer_size=4096)
     decoded = io.TextIOWrapper(buffered, encoding=encoding)
-    if pk and len(pk) == 1:
-        pk = pk[0]
+    tracker = None
     if csv or tsv:
         if sniff:
             # Read first 2048 bytes and use that to detect
@@ -720,6 +748,9 @@ def insert_upsert_implementation(
             else:
                 headers = first_row
             docs = (dict(zip(headers, row)) for row in reader)
+            if detect_types:
+                tracker = TypeTracker()
+                docs = tracker.wrap(docs)
     else:
         try:
             if nl:
@@ -752,6 +783,8 @@ def insert_upsert_implementation(
                 "{}\n\nTry using --alter to add additional columns".format(e.args[0])
             )
         raise
+    if tracker is not None:
+        db[table].transform(types=tracker.types)
 
 
 @cli.command()
@@ -786,6 +819,7 @@ def insert(
     batch_size,
     alter,
     encoding,
+    detect_types,
     load_extension,
     silent,
     ignore,
@@ -820,6 +854,7 @@ def insert(
             replace=replace,
             truncate=truncate,
             encoding=encoding,
+            detect_types=detect_types,
             load_extension=load_extension,
             silent=silent,
             not_null=not_null,
@@ -848,6 +883,7 @@ def upsert(
     not_null,
     default,
     encoding,
+    detect_types,
     load_extension,
     silent,
 ):
@@ -1087,8 +1123,168 @@ def query(
         db.attach(alias, attach_path)
     _load_extensions(db, load_extension)
     db.register_fts4_bm25()
+
+    _execute_query(
+        db, sql, param, raw, table, csv, tsv, no_headers, fmt, nl, arrays, json_cols
+    )
+
+
+@cli.command()
+@click.argument(
+    "paths",
+    type=click.Path(file_okay=True, dir_okay=False, allow_dash=True),
+    required=False,
+    nargs=-1,
+)
+@click.argument("sql")
+@click.option(
+    "--attach",
+    type=(str, click.Path(file_okay=True, dir_okay=False, allow_dash=False)),
+    multiple=True,
+    help="Additional databases to attach - specify alias and filepath",
+)
+@output_options
+@click.option("-r", "--raw", is_flag=True, help="Raw output, first column of first row")
+@click.option(
+    "-p",
+    "--param",
+    multiple=True,
+    type=(str, str),
+    help="Named :parameters for SQL query",
+)
+@click.option(
+    "--encoding",
+    help="Character encoding for CSV input, defaults to utf-8",
+)
+@click.option(
+    "-n",
+    "--no-detect-types",
+    is_flag=True,
+    help="Treat all CSV/TSV columns as TEXT",
+)
+@click.option("--schema", is_flag=True, help="Show SQL schema for in-memory database")
+@click.option("--dump", is_flag=True, help="Dump SQL for in-memory database")
+@click.option(
+    "--save",
+    type=click.Path(file_okay=True, dir_okay=False, allow_dash=False),
+    help="Save in-memory database to this file",
+)
+@load_extension_option
+def memory(
+    paths,
+    sql,
+    attach,
+    nl,
+    arrays,
+    csv,
+    tsv,
+    no_headers,
+    table,
+    fmt,
+    json_cols,
+    raw,
+    param,
+    encoding,
+    no_detect_types,
+    schema,
+    dump,
+    save,
+    load_extension,
+):
+    """Execute SQL query against an in-memory database, optionally populated by imported data
+
+    To import data from CSV, TSV or JSON files pass them on the command-line:
+
+    \b
+        sqlite-utils memory one.csv two.json \\
+            "select * from one join two on one.two_id = two.id"
+
+    For data piped into the tool from standard input, use "-" or "stdin":
+
+    \b
+        cat animals.csv | sqlite-utils memory - \\
+            "select * from stdin where species = 'dog'"
+
+    The format of the data will be automatically detected. You can specify the format
+    explicitly using :json, :csv, :tsv or :nl (for newline-delimited JSON) - for example:
+
+    \b
+        cat animals.csv | sqlite-utils memory stdin:csv places.dat:nl \\
+            "select * from stdin where place_id in (select id from places)"
+
+    Use --schema to view the SQL schema of any imported files:
+
+    \b
+        sqlite-utils memory animals.csv --schema
+    """
+    db = sqlite_utils.Database(memory=True)
+    # If --dump or --save used but no paths detected, assume SQL query is a path:
+    if (dump or save or schema) and not paths:
+        paths = [sql]
+        sql = None
+    for i, path in enumerate(paths):
+        # Path may have a :format suffix
+        if ":" in path and path.rsplit(":", 1)[-1].upper() in Format.__members__:
+            path, suffix = path.rsplit(":", 1)
+            format = Format[suffix.upper()]
+        else:
+            format = None
+        if path in ("-", "stdin"):
+            csv_fp = sys.stdin.buffer
+            csv_table = "stdin"
+        else:
+            csv_path = pathlib.Path(path)
+            csv_table = csv_path.stem
+            csv_fp = csv_path.open("rb")
+        rows, format_used = rows_from_file(csv_fp, format=format, encoding=encoding)
+        tracker = None
+        if format_used in (Format.CSV, Format.TSV) and not no_detect_types:
+            tracker = TypeTracker()
+            rows = tracker.wrap(rows)
+        db[csv_table].insert_all(rows, alter=True)
+        if tracker is not None:
+            db[csv_table].transform(types=tracker.types)
+        # Add convenient t / t1 / t2 views
+        view_names = ["t{}".format(i + 1)]
+        if i == 0:
+            view_names.append("t")
+        for view_name in view_names:
+            if not db[view_name].exists():
+                db.create_view(view_name, "select * from [{}]".format(csv_table))
+
+    if dump:
+        for line in db.conn.iterdump():
+            click.echo(line)
+        return
+
+    if schema:
+        click.echo(db.schema)
+        return
+
+    if save:
+        db2 = sqlite_utils.Database(save)
+        for line in db.conn.iterdump():
+            db2.execute(line)
+        return
+
+    for alias, attach_path in attach:
+        db.attach(alias, attach_path)
+    _load_extensions(db, load_extension)
+    db.register_fts4_bm25()
+
+    _execute_query(
+        db, sql, param, raw, table, csv, tsv, no_headers, fmt, nl, arrays, json_cols
+    )
+
+
+def _execute_query(
+    db, sql, param, raw, table, csv, tsv, no_headers, fmt, nl, arrays, json_cols
+):
     with db.conn:
-        cursor = db.execute(sql, dict(param))
+        try:
+            cursor = db.execute(sql, dict(param))
+        except sqlite3.OperationalError as e:
+            raise click.ClickException(str(e))
         if cursor.description is None:
             # This was an update/insert
             headers = ["rows_affected"]
@@ -1287,6 +1483,90 @@ def triggers(
     )
 
 
+@cli.command()
+@click.argument(
+    "path",
+    type=click.Path(file_okay=True, dir_okay=False, allow_dash=False),
+    required=True,
+)
+@click.argument("tables", nargs=-1)
+@click.option("--aux", is_flag=True, help="Include auxiliary columns")
+@output_options
+@load_extension_option
+@click.pass_context
+def indexes(
+    ctx,
+    path,
+    tables,
+    aux,
+    nl,
+    arrays,
+    csv,
+    tsv,
+    no_headers,
+    table,
+    fmt,
+    json_cols,
+    load_extension,
+):
+    "Show indexes for this database"
+    sql = """
+    select
+      sqlite_master.name as "table",
+      indexes.name as index_name,
+      xinfo.*
+    from sqlite_master
+      join pragma_index_list(sqlite_master.name) indexes
+      join pragma_index_xinfo(index_name) xinfo
+    where
+      sqlite_master.type = 'table'
+    """
+    if tables:
+        quote = sqlite_utils.Database(memory=True).quote
+        sql += " and sqlite_master.name in ({})".format(
+            ", ".join(quote(table) for table in tables)
+        )
+    if not aux:
+        sql += " and xinfo.key = 1"
+    ctx.invoke(
+        query,
+        path=path,
+        sql=sql,
+        nl=nl,
+        arrays=arrays,
+        csv=csv,
+        tsv=tsv,
+        no_headers=no_headers,
+        table=table,
+        fmt=fmt,
+        json_cols=json_cols,
+        load_extension=load_extension,
+    )
+
+
+@cli.command()
+@click.argument(
+    "path",
+    type=click.Path(file_okay=True, dir_okay=False, allow_dash=False),
+    required=True,
+)
+@click.argument("tables", nargs=-1, required=False)
+@load_extension_option
+def schema(
+    path,
+    tables,
+    load_extension,
+):
+    "Show full schema for this database or for specified tables"
+    db = sqlite_utils.Database(path)
+    _load_extensions(db, load_extension)
+    if tables:
+        for table in tables:
+            click.echo(db[table].schema)
+    else:
+        click.echo(db.schema)
+
+
 @cli.command()
 @click.argument(
     "path",
