@@ -1,5 +1,5 @@
 from .utils import sqlite3, OperationalError, suggest_column_types, column_affinity
-from collections import namedtuple, OrderedDict
+from collections import namedtuple
 from collections.abc import Mapping
 import contextlib
 import datetime
@@ -11,9 +11,10 @@ import json
 import os
 import pathlib
 import re
-from sqlite_fts4 import rank_bm25
+from sqlite_fts4 import rank_bm25  # type: ignore
 import sys
 import textwrap
+from typing import Generator, Iterable, Union, Optional, List
 import uuid
 
 SQLITE_MAX_VARS = 999
@@ -39,14 +40,14 @@ USING\s+(?P<using>\w+)          # e.g. USING FTS5
 )
 
 try:
-    import pandas as pd
+    import pandas as pd  # type: ignore
 except ImportError:
     pd = None
 
 try:
-    import numpy as np
+    import numpy as np  # type: ignore
 except ImportError:
-    np = None
+    np = None  # type: ignore
 
 Column = namedtuple(
     "Column", ("cid", "name", "type", "notnull", "default_value", "is_pk")
@@ -68,6 +69,10 @@ ForeignKey = namedtuple(
     "ForeignKey", ("table", "column", "other_table", "other_column")
 )
 Index = namedtuple("Index", ("seq", "name", "unique", "origin", "partial", "columns"))
+XIndex = namedtuple("XIndex", ("name", "columns"))
+XIndexColumn = namedtuple(
+    "XIndexColumn", ("seqno", "cid", "name", "desc", "coll", "key")
+)
 Trigger = namedtuple("Trigger", ("name", "table", "sql"))
 
 
@@ -297,6 +302,18 @@ class Database:
         "Returns {trigger_name: sql} dictionary"
         return {trigger.name: trigger.sql for trigger in self.triggers}
 
+    @property
+    def schema(self):
+        sqls = []
+        for row in self.execute(
+            "select sql from sqlite_master where sql is not null"
+        ).fetchall():
+            sql = row[0]
+            if not sql.strip().endswith(";"):
+                sql += ";"
+            sqls.append(sql)
+        return "\n".join(sqls)
+
     @property
     def journal_mode(self):
         return self.execute("PRAGMA journal_mode;").fetchone()[0]
@@ -343,10 +360,18 @@ class Database:
                 for table in tables
             )
 
-    def execute_returning_dicts(self, sql, params=None):
+    def query(
+        self, sql: str, params: Optional[Union[Iterable, dict]] = None
+    ) -> Generator[dict, None, None]:
         cursor = self.execute(sql, params or tuple())
         keys = [d[0] for d in cursor.description]
-        return [dict(zip(keys, row)) for row in cursor.fetchall()]
+        for row in cursor:
+            yield dict(zip(keys, row))
+
+    def execute_returning_dicts(
+        self, sql: str, params: Optional[Union[Iterable, dict]] = None
+    ) -> List[dict]:
+        return list(self.query(sql, params))
 
     def resolve_foreign_keys(self, name, foreign_keys):
         # foreign_keys may be a list of strcolumn names, a list of ForeignKey tuples,
@@ -793,6 +818,10 @@ class Table(Queryable):
             names = ["rowid"]
         return names
 
+    @property
+    def use_rowid(self):
+        return not any(column for column in self.columns if column.is_pk)
+
     def get(self, pk_values):
         if not isinstance(pk_values, (list, tuple)):
             pk_values = [pk_values]
@@ -863,6 +892,24 @@ class Table(Queryable):
             indexes.append(Index(**row))
         return indexes
 
+    @property
+    def xindexes(self):
+        sql = 'PRAGMA index_list("{}")'.format(self.name)
+        indexes = []
+        for row in self.db.execute_returning_dicts(sql):
+            index_name = row["name"]
+            index_name_quoted = (
+                '"{}"'.format(index_name)
+                if not index_name.startswith('"')
+                else index_name
+            )
+            column_sql = "PRAGMA index_xinfo({})".format(index_name_quoted)
+            index_columns = []
+            for info in self.db.execute(column_sql).fetchall():
+                index_columns.append(XIndexColumn(*info))
+            indexes.append(XIndex(index_name, index_columns))
+        return indexes
+
     @property
     def triggers(self):
         return [
@@ -978,7 +1025,9 @@ class Table(Queryable):
 
         sqls = []
         if pk is DEFAULT:
-            pks_renamed = tuple(rename.get(p) or p for p in self.pks)
+            pks_renamed = tuple(
+                rename.get(p.name) or p.name for p in self.columns if p.is_pk
+            )
             if len(pks_renamed) == 1:
                 pk = pks_renamed[0]
             else:
@@ -1072,8 +1121,6 @@ class Table(Queryable):
                 )
             )
         table = table or "_".join(columns)
-        first_column = columns[0]
-        pks = self.pks
         lookup_table = self.db[table]
         fk_column = fk_column or "{}_id".format(table)
         magic_lookup_column = "{}_{}".format(fk_column, os.urandom(6).hex())
@@ -1192,7 +1239,7 @@ class Table(Queryable):
         fk_col_type = None
         if fk is not None:
             # fk must be a valid table
-            if not fk in self.db.table_names():
+            if fk not in self.db.table_names():
                 raise AlterError("table '{}' does not exist".format(fk))
             # if fk_col specified, must be a valid column
             if fk_col is not None:
@@ -1698,20 +1745,22 @@ class Table(Queryable):
                 queries_and_params.append((sql, [record[col] for col in pks]))
                 # UPDATE [book] SET [name] = 'Programming' WHERE [id] = 1001;
                 set_cols = [col for col in all_columns if col not in pks]
-                sql2 = "UPDATE [{table}] SET {pairs} WHERE {wheres}".format(
-                    table=self.name,
-                    pairs=", ".join(
-                        "[{}] = {}".format(col, conversions.get(col, "?"))
-                        for col in set_cols
-                    ),
-                    wheres=" AND ".join("[{}] = ?".format(pk) for pk in pks),
-                )
-                queries_and_params.append(
-                    (
-                        sql2,
-                        [record[col] for col in set_cols] + [record[pk] for pk in pks],
+                if set_cols:
+                    sql2 = "UPDATE [{table}] SET {pairs} WHERE {wheres}".format(
+                        table=self.name,
+                        pairs=", ".join(
+                            "[{}] = {}".format(col, conversions.get(col, "?"))
+                            for col in set_cols
+                        ),
+                        wheres=" AND ".join("[{}] = ?".format(pk) for pk in pks),
+                    )
+                    queries_and_params.append(
+                        (
+                            sql2,
+                            [record[col] for col in set_cols]
+                            + [record[pk] for pk in pks],
+                        )
                     )
-                )
                 # We can populate .last_pk right here
                 if num_records_processed == 1:
                     self.last_pk = tuple(record[pk] for pk in pks)
